#!/usr/local/bin/python2.7

# https://fluxtream.org/dev/#/api-docs

from flask import Flask, abort, request, redirect, send_file, make_response, render_template, Response, abort
from flask.ext.restful import abort, Api, Resource, reqparse, fields, marshal_with
from flask_restful_swagger import swagger
from flask.ext.httpauth import HTTPBasicAuth
from flask.ext.cache import Cache
import os 
app = Flask(__name__)
app.config['SECRET_KEY'] = 't92%UP7rVM$bzJ$GzYx7Ohm0'
# cache = Cache(app,config={'CACHE_TYPE': 'simple'})

CACHE_SHORT_TIMEOUT = 300*2
CACHE_DAILY_TIMEOUT = 86400

app.config['CACHE_TYPE'] = 'filesystem'
app.config['CACHE_DEFAULT_TIMEOUT'] = CACHE_SHORT_TIMEOUT
app.config['CACHE_DIR'] = os.path.dirname(os.path.realpath(__file__)) + "/cache"
app.config['CACHE_THRESHOLD'] = 5000
app.cache = Cache(app)

auth = HTTPBasicAuth()

import ws.carre as carre
import virtuoso

api = swagger.docs(Api(app), apiVersion='0.1',
				   basePath='http://localhost:5000',
				   resourcePath='/',
				   produces=["application/json", "text/html"],
				   description = 'CARRE web services API',
				   api_spec_url='/api/spec')


@app.errorhandler(404)
def page_not_found(error):
	return "page not found. bye"
	return render_template('page_not_found.html'), 404

@auth.verify_password
def verify_password(username, password):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST
		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sql = "SELECT ?subject ?predicate ?object FROM <https://carre.kmi.open.ac.uk/users/" + username + "> WHERE { ?subject ?predicate ?object } limit 1"
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)
		sparql.setQuery(sql)
		results = sparql.query().convert()
		return True
	except Exception, e:
		return False
	return True

class DailyMeasurement(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		self.reqparse.add_argument('day', required=True, type=str, help='Day(YYYY-MM-DD)')
		return


	@swagger.operation(
		notes = "Provides daily measurements",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "day",
			"description": "Date in the format YYYY-MM-DD",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			},
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import json
			args = self.reqparse.parse_args()
			day = args['day']
			token = args['token']
			rs = carre.__getDailyMeasurements__(token=token,d=day)
			return json.loads(rs.to_json(orient='records',date_format='iso'))
		except Exception, e:
			return str(e)

class Measurement(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('name', required=True, type=str, help='Measurement name')
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return


	@swagger.operation(
		notes = "Provides measurements",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "name",
			"description": "Name of the measurement",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			},
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import json
			args = self.reqparse.parse_args()
			measurement = args['name']
			token = args['token']
			rs = carre.getDataContinuous(measurement, token=token)
			if isinstance(rs, str):
				return Response(json.dumps("invalid measurement "  + measurement))
			# return Response(rs.to_json(orient='records',date_format='iso'))
			return json.loads(rs.to_json(orient='records',date_format='iso'))
		except Exception, e:
			return Response("invalid measurement name")
			return str(e)

class MeasurementManufacturer(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('manufacturer', required=True, type=str, help='Manufacturer name')
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return


	@swagger.operation(
		notes = "Provides all measurements by one manufacturer",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "manufacturer",
			"description": "Name of the measurement",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			},
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import json
			args = self.reqparse.parse_args()
			manufacturer = args['manufacturer']
			token = args['token']
			manufacturer = args['manufacturer']
			rs = carre.getMeasurementManufacturer(manufacturer, token)
			return json.dumps(rs)
		except Exception, e:
			return Response("invalid input")

class MeasurementsList(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return

	@swagger.operation(
		notes = "Provides available measurements",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import json
			args = self.reqparse.parse_args()
			token = args['token']
			rs = carre.__listAvailableMeasurements2__(token=token)
			if isinstance(rs, str):
				return Response(json.dumps("invalid request"))

			return json.loads(rs.to_json(orient='records',date_format='iso'))
		except Exception, e:
			abort(500)
			return Response("invalid token")
			return str(e)

class UserProfile(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return


	@swagger.operation(
		notes = "Provides user profile information",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			args = self.reqparse.parse_args()
			token = args['token']
			rs = {}
			rs['username'] = virtuoso.getUsernameFromToken(token)
			if rs['username'] is None:
				return {'error':'invalid token'}
			rs['email'] = virtuoso.getEmailByUsername(rs['username'])
			rs['graphName'] = "https://carre.kmi.open.ac.uk/users/" + rs['username']
			return rs
		except Exception, e:
			return str(e)

class UserManufacturer(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return


	@swagger.operation(
		notes = "Provides manufacturers providing data for a user",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[
			{
			"name": "token",
			"description": "OAuth token",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
			}],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import json
			args = self.reqparse.parse_args()
			token = args['token']
			rs = carre.getUserManufacturers(token)
			return rs
		except Exception, e:
			return str(e)

class Token(Resource):
	@auth.login_required
	@swagger.operation(
		notes = "Provides authentication tokens. Basic authentication is required.",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		import ws.virtuoso as virtuoso
		up = request.headers.get('Authorization').replace("Basic ","").decode("base64")
		username = up.split(':', 1)[0]
		password = up.split(':', 1)[1]

		rs = virtuoso.authenticate(username, password)
		if isinstance(rs, basestring):
			return Response("False")
		d = {}
		d['token'] = rs[0]

		import json
		return Response(json.dumps(d),mimetype="application/json")

class CalendarByToken(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return

	@swagger.operation(
		notes = "Provides calendar, also accessible through <a href='webcal://carre.kmi.open.ac.uk/ws/icalToken.ics?token=XXX'>webcal://carre.kmi.open.ac.uk/ws/icalToken.ics?token=XXX</a>.",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
			"name": "token",
			"description": "Authentication token",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import ws.virtuoso as virtuoso
			self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
			args = self.reqparse.parse_args()
			token = args['token']
			return Response(carre.createICal(token=token), mimetype='text/calendar')
		except Exception, e:
			return Response("invalid token: "+str(e))

class Calendar(Resource):
	@auth.login_required
	@swagger.operation(
		notes = "Provides calendar, also accessible through <a href='webcal://carre.kmi.open.ac.uk/ws/ical.ics'>webcal://carre.kmi.open.ac.uk/ws/ical.ics</a>. Basic authentication is required. ",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	def get(self):
		try:
			import ws.virtuoso as virtuoso
			up = request.headers.get('Authorization').replace("Basic ","").decode("base64")
			username = up.split(':', 1)[0]
			password = up.split(':', 1)[1]
			return Response(carre.createICal(username,password), mimetype='text/calendar')
		except Exception,e:
			return Response(str(e))

class Instances(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('type', required=True, type=str, help='Instance type')
		return

	@swagger.operation(
		notes = "Fetches instances. Choose from observable, clinical_observable, personal_observable, risk_element, biomedical_risk_element, behavioural_risk_element, genetic_risk_element, demographic_risk_element, risk_factor, risk_evidence, citation",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
			"name": "type",
			"description": "Instance type.",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	@app.cache.memoize(timeout=CACHE_SHORT_TIMEOUT)
	def get(self):
		try:
			self.reqparse.add_argument('type', required=True, type=str, help='Instance type')
			args = self.reqparse.parse_args()
			t = args['type']
			query = """SELECT ?subject ?predicate ?object FROM <http://carre.kmi.open.ac.uk/beta> WHERE {
						?subject <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://carre.kmi.open.ac.uk/ontology/risk.owl#"""+t+"""> .
						?subject ?predicate ?object .
						}"""
			import json
			rs = carre.sparql(query, "dba", "DBA_PASSWORD_HERE")
			data = json.loads(rs)
			data2 = []
			for record in data:
				data2.append({'subject':record['subject']['value'],'predicate':record['predicate']['value'],'object':record['object']['value']})
			return Response(json.dumps(data2))
		except Exception,e:
			return Response(str(e))

class RiskEvidencePublications(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('IRI', required=True, type=str, help='Risk Evidence IRI')
		return

	@swagger.operation(
		notes = "Fetches citations for a given risk evidence, e.g. http://carre.kmi.open.ac.uk/CARRE_RE225. See /instances for a list of Risk Evidence.",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
			"name": "IRI",
			"description": "IRI of the Risk Evidence.",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	@app.cache.memoize(timeout=CACHE_SHORT_TIMEOUT)
	def get(self):
		try:
			self.reqparse.add_argument('IRI', required=True, type=str, help='Risk Evidence IRI')
			args = self.reqparse.parse_args()
			iri = args['IRI']
			import json
			rs = carre.getRiskEvidenceCitations(iri)
			return Response(json.dumps(rs))
		except Exception,e:
			return Response(str(e))

class PUBMedPublication(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('id', required=True, type=str, help='Risk Evidence IRI')
		return

	@swagger.operation(
		notes = "Fetches metadata for a certain PUBMed publication.",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
			"name": "id",
			"description": "PUBMed id of the publication.",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	@app.cache.memoize(timeout=CACHE_SHORT_TIMEOUT)
	def get(self):
		try:
			self.reqparse.add_argument('id', required=True, type=str, help='PUBMed id')
			args = self.reqparse.parse_args()
			id = args['id']
			import json
			rs = carre.getPUBMedMetadata(id)
			return Response(json.dumps(rs))
		except Exception,e:
			return Response(str(e))

class UMLsCUI(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('cui', required=True, type=str, help='UMLs CUI')
		return

	@swagger.operation(
		notes = "Fetches metadata for a UMLs entry.",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
			"name": "cui",
			"description": "Fetches metadata for a UMLs entry.",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	@app.cache.memoize(timeout=CACHE_SHORT_TIMEOUT)
	def get(self):
		try:
			self.reqparse.add_argument('cui', required=True, type=str, help='PUBMed id')
			args = self.reqparse.parse_args()
			cui = args['cui']
			import json
			rs = carre.getUMLsCUI(cui)
			return Response(json.dumps(rs))
		except Exception,e:
			return Response(str(e))

class BioPortalUnitDesc(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('uri', required=True, type=str, help='URI of individual reading type')
		return

	@swagger.operation(
		notes = "Fetches BioPortal Unit Description. E.g., http://carre.kmi.open.ac.uk/ontology/sensors.owl#individual_sleep_measurement",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
			"name": "uri",
			"description": "URI of individual reading type.",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	@app.cache.memoize(timeout=CACHE_SHORT_TIMEOUT)
	def get(self):
		try:
			self.reqparse.add_argument('uri', required=True, type=str, help='URI of individual reading type')
			args = self.reqparse.parse_args()
			uri = args['uri']
			import json
			rs = carre.getBioPortalUnitDesc(uri)
			return Response(json.dumps(rs))
		except Exception,e:
			return Response(str(e))

class BioPortalMeasurementDesc(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('uri', required=True, type=str, help='URI of individual reading type')
		return

	@swagger.operation(
		notes = "Fetches BioPortal Measurement Description. E.g., http://carre.kmi.open.ac.uk/ontology/sensors.owl#individual_sleep_measurement",
		responseClass = "JSON",
		nickname = "nickname",
		parameters=[{
			"name": "uri",
			"description": "URI of individual reading type.",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }],
			responseMessages=[
			{
				"code": 201,
				"message": "Executed."
			},
			{
				"code": 405,
				"message": "Invalid input"
			}]
		)
	@app.cache.memoize(timeout=CACHE_SHORT_TIMEOUT)
	def get(self):
		try:
			self.reqparse.add_argument('uri', required=True, type=str, help='URI of individual reading type')
			args = self.reqparse.parse_args()
			uri = args['uri']
			import json
			rs = carre.getBioPortalMeasurementDesc(uri)
			return Response(json.dumps(rs))
		except Exception,e:
			return Response(str(e))

class Query(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('sparql', required=True, type=str, help='SPARQL query')
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return

	# @auth.login_required
	@swagger.operation(
	  notes='This is the simplest implementation possible. A SPARQL query is passed to the Virtuoso server and results are returned directly to the client.',
	  responseClass="String in the desired format. JSON (default), XML, TURTLE, N3, RDF",#ModelClass.__name__,
	  nickname='upload',
	  parameters=[
		  {
			"name": "sparql",
			"description": """SPARQL statement to be passed to the RDF repository.""",
								#E.g. SELECT ?subject ?predicate ?object FROM <http://carre.kmi.open.ac.uk/users/gkotsis> WHERE { ?subject ?predicate ?object } limit 1""",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
		  },
		  {
			"name": "token",
			"description": "Authentication token",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }

		],
	  responseMessages=[
		  {
			"code": 201,
			"message": "Executed."
		  },
		  {
			"code": 405,
			"message": "Invalid input"
		  }
		]
	  )
	def post(self):
		try:
			self.reqparse.add_argument('sparql', required=True, type=str, help='<sparql> query must be passed')
			self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
			args = self.reqparse.parse_args()
			sparql = args['sparql']
			token = args['token']
			rs = virtuoso.executeSPARQL(sparql, token)
			if isinstance(rs, str):
				return {'error':str(type(rs))}
			else:
				return rs

		except Exception, e:
			return str(e)

class User(Resource):
	def __init__(self):
		self.reqparse = reqparse.RequestParser()
		self.reqparse.add_argument('username', required=True, type=str, help='username')
		self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
		return

	@swagger.operation(
	  notes='Method for deleting a user',
	  responseClass="Error message if any",#ModelClass.__name__,
	  nickname='upload',
	  parameters=[
		  {
			"name": "username",
			"description": """Username.""",
								#E.g. SELECT ?subject ?predicate ?object FROM <http://carre.kmi.open.ac.uk/users/gkotsis> WHERE { ?subject ?predicate ?object } limit 1""",
			"required": True,
			"allowMultiple": False,
			"dataType": "String",
			"paramType": "query",
		  },
		  {
			"name": "token",
			"description": "Authentication token",
			"required": True,
			"default": "JSON",
			"allowMultiple": False,
			"dataType": 'String',
			"paramType": "query"
		  }

		],
	  responseMessages=[
		  {
			"code": 201,
			"message": "Executed."
		  },
		  {
			"code": 405,
			"message": "Invalid input"
		  }
		]
	  )
	def post(self):
		try:
			self.reqparse.add_argument('username', required=True, type=str, help='<username> query must be passed')
			self.reqparse.add_argument('token', required=True, type=str, help='OAuth token')
			args = self.reqparse.parse_args()
			username = args['username']
			token = args['token']
			return virtuoso.deleteUser(username, token)
		except Exception, e:
			return str(e)

@app.route('/docs')
def docs():
  return redirect('/ws/api/spec.html')



@app.route('/dereference/<path:url>')
@auth.login_required
def dereference(url):
	import carre
	up = request.headers.get('Authorization').replace("Basic ","").decode("base64")
	username = up.split(':', 1)[0]
	password = up.split(':', 1)[1]
	if "http:/carre" in url:
		url = url.replace("http:/carre", "http://carre")
	if "https:/carre" in url:
		url = url.replace("https:/carre", "https://carre")
	
	format = "N3"
	if request.headers['Content-Type'] == 'text/plain':
		format = "N3"
	if request.headers['Content-Type'] == 'application/json':
		format = "JSON"
		import json
		rs = carre.describe(url, username, password, returnFormat=format)
		return Response(json.dumps(rs,indent=2), mimetype='application/json')
	if request.headers['Content-Type'] == 'application/xml':
		format = "XML"
		rs = carre.describe(url, username, password, returnFormat=format)
		rs = rs.serialize()
		return Response(rs, mimetype='application/xml')

	# return Response(request.headers['Content-Type'])
	return Response(carre.describe(url, username, password, returnFormat=format))


api.add_resource(Query, '/query')
api.add_resource(User, '/deleteUser')
api.add_resource(Measurement, '/measurement')
api.add_resource(MeasurementManufacturer, '/measurementManufacturer')
api.add_resource(MeasurementsList, '/measurementsList')
api.add_resource(DailyMeasurement, '/dailyMeasurement')
api.add_resource(UserProfile, '/userProfile')
api.add_resource(UserManufacturer, '/userManufacturer')
api.add_resource(Token, '/authenticate')
api.add_resource(Calendar, '/ical.ics')
api.add_resource(CalendarByToken, '/icalToken.ics')
api.add_resource(Instances, '/instances')
api.add_resource(RiskEvidencePublications, '/riskEvidencePublications')
api.add_resource(PUBMedPublication, '/PUBMedPublication')
api.add_resource(UMLsCUI, '/UMLsCUI')
api.add_resource(BioPortalUnitDesc, '/BioPortalUnitDesc')
api.add_resource(BioPortalMeasurementDesc, '/BioPortalMeasurementDesc')
# parser = reqparse.RequestParser()


@app.route('/')
def index():
  return redirect('/ws/api/spec.html')


if __name__ == '__main__':
	app.debug = True
	Query()
	app.run(debug=True, port=65010)
