#!/usr/local/bin/python2.7

from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF

def login(username, password):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST
		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sql = "SELECT ?subject ?predicate ?object FROM <http://carre.kmi.open.ac.uk/users/" + username + "> WHERE { ?subject ?predicate ?object } limit 1"
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)
		sparql.setQuery(sql)
		results = sparql.query().convert()
		return True
	except Exception, e:
		return False
	return True

settings = {}
settings['carreSensorsOntologyURL'] = "http://carre.kmi.open.ac.uk/ontology/sensors.owl" #not used directly

measurements = {}
measurements['Calories'] = settings['carreSensorsOntologyURL'] + '#has_calories'
measurements['Date'] = settings['carreSensorsOntologyURL'] + '#has_date'
measurements['CaloriesBMR'] = settings['carreSensorsOntologyURL'] + '#has_calories_b_m_r'
measurements['ActivityCalories'] = settings['carreSensorsOntologyURL'] + '#has_activity_calories'
measurements['MarginalCalories'] = settings['carreSensorsOntologyURL'] + '#has_marginal_calories'
measurements['SedentaryActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_sedentary_activity_duration'
measurements['LightActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_light_activity_duration'
measurements['StartDate'] = settings['carreSensorsOntologyURL'] + '#has_start_date'
measurements['SleepStatus'] = settings['carreSensorsOntologyURL'] + '#has_sleep_status'
measurements['Steps'] = settings['carreSensorsOntologyURL'] + '#has_steps'
measurements['Distance'] = settings['carreSensorsOntologyURL'] + '#has_distance'
measurements['Elevation'] = settings['carreSensorsOntologyURL'] + '#has_elevation'
measurements['Floors'] = settings['carreSensorsOntologyURL'] + '#has_floors'
measurements['ModerateActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_moderate_activity_duration'
measurements['LightActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_light_activity_distance'
measurements['ModerateActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_moderate_activity_distance'
measurements['TrackedActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_tracked_activity_distance'
measurements['IntenseActivityDuration'] = settings['carreSensorsOntologyURL'] + '#has_intense_activity_duration'
measurements['IntenseActivityDistance'] = settings['carreSensorsOntologyURL'] + '#has_intense_activity_distance'
measurements['TimesAwake'] = settings['carreSensorsOntologyURL'] + '#has_times_awake'
measurements['TimesRestless'] = settings['carreSensorsOntologyURL'] + '#has_times_restless'
measurements['TimeToFallAsleep'] = settings['carreSensorsOntologyURL'] + '#has_time_to_fall_asleep'
measurements['SleepTime'] = settings['carreSensorsOntologyURL'] + '#has_sleep_time'
measurements['SleepEfficiency'] = settings['carreSensorsOntologyURL'] + '#has_sleep_efficiency'
measurements['AwakeDuration'] = settings['carreSensorsOntologyURL'] + '#has_awake_duration'
measurements['AsleepDuration'] = settings['carreSensorsOntologyURL'] + '#has_asleep_duration'
measurements['RestlessDuration'] = settings['carreSensorsOntologyURL'] + '#has_restless_duration'
measurements['MinutesAfterWakeup'] = settings['carreSensorsOntologyURL'] + '#has_minutes_after_wakeup'
measurements['SystolicBloodPressure'] = settings['carreSensorsOntologyURL'] + "#has_blood_pressure_systolic"
measurements['DiastolicBloodPressure'] = settings['carreSensorsOntologyURL'] + "#has_blood_pressure_diastolic"
measurements['Pulse'] = settings['carreSensorsOntologyURL'] + "#has_pulse"
measurements['OxygenSaturation'] = settings['carreSensorsOntologyURL'] + "#has_o2saturation"

def __getQuery__(predicate, startDate ,username):
	sql = """
		SELECT max(?v1) as ?value xsd:date(?d) as ?d FROM <https://carre.kmi.open.ac.uk/users/""" + username + """> WHERE
		{
		         ?subject ?predicate ?object.
		         ?subject <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date> ?d1.
		         ?d1 <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?d.
                         ?subject <"""+predicate+"""> ?v.
                         ?v <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?v1.
		         FILTER ( ( xsd:date(?d) >= '""" + startDate + """'^^xsd:date) )
		}
		GROUP BY xsd:date(?d)
	"""
	return sql

def __getRiskEvidenceCitationsQuery__(IRI):
	sql = """
		SELECT ?o FROM <http://carre.kmi.open.ac.uk/beta> WHERE {
			<""" + IRI + """> <http://carre.kmi.open.ac.uk/ontology/risk.owl#has_risk_evidence_source> ?o
		} 
	"""
	return sql

def getStartDate(period=365):
	import pandas as pd
	from datetime import date, timedelta
	d = pd.datetime.today() - timedelta(days=period)
	d = str(d.year) + d.strftime('%m') + d.strftime('%d')
	return d

def getPredicate(measurement):
	global measurements
	predicate = None
	if measurement in measurements.keys():
			predicate = measurements[measurement]
	elif measurement in measurements.values():
		predicate = measurement

	return predicate

def getPUBMedMetadata(id):
	import pubmed
	d = pubmed.getPUBMEDByID(id)
	return d
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
	sql = """SELECT *
			WHERE {
			    <http://linkedlifedata.com/resource/pubmed/id/""" + id + """> ?p ?o .
			}"""
	lld = SPARQLWrapper("http://linkedlifedata.com/sparql.srj")
	lld.setQuery(sql)
	lld.setReturnFormat(JSON)
	results = lld.query().convert()
	d = {}
	for result in results['results']['bindings']:
		if result['p']['value'] in d.keys():
			d[result['p']['value']] = d[result['p']['value']] + "," + result['o']['value']
		else:
			d[result['p']['value']] = result['o']['value']
	return d

def getUMLsCUI(cui):
	cui = cui.upper()
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
	sql = """SELECT *
			WHERE {
			    <http://linkedlifedata.com/resource/umls/id/""" + cui + """> ?p ?o .
			}"""
	lld = SPARQLWrapper("http://linkedlifedata.com/sparql.srj")
	lld.setQuery(sql)
	lld.setReturnFormat(JSON)
	results = lld.query().convert()
	d = {}
	for result in results['results']['bindings']:
		if result['p']['value'] in d.keys():
			d[result['p']['value']] = d[result['p']['value']] + "," + result['o']['value']
		else:
			d[result['p']['value']] = result['o']['value']
	return d

def getBioPortalUnitDesc(uri):
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
	sql = """PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
			PREFIX owl: <http://www.w3.org/2002/07/owl#>
			PREFIX : <http://carre.kmi.open.ac.uk/ontology/sensors.owl#>


			SELECT ?cType ?s ?p  ?p2 ?o WHERE 
			{ 
			   GRAPH  <http://carre.kmi.open.ac.uk/beta> {
			   ?measurementPredicate rdfs:domain <""" + uri + """> . 
			    ?measurementPredicate rdfs:range ?measurementTypeValue .
			    ?measurementTypeValue rdfs:subClassOf ?value . 
			     ?value rdfs:subClassOf ?entity . 
			      ?entity owl:onProperty :has_unit  .
			     ?entity owl:someValuesFrom ?unit . 
			    ?unit rdfs:subClassOf ?cType . 
			 }
			SERVICE <http://sparql.bioontology.org/ontologies/sparql?apikey=51d0c12d-9c18-4401-b85c-7ea4061fc0c2>
			   { 
				GRAPH ?g {
			         OPTIONAL {?s ?p ?cType }.
			         OPTIONAL { ?cType ?p2 ?o }. 
			         OPTIONAL {?s2 ?cType ?o2 }. 
				}
			   }
			} GROUP BY ?cType"""
	lld = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql")
	lld.setQuery(sql)
	lld.setReturnFormat(JSON)
	results = lld.query().convert()
	rs = []
	for result in results['results']['bindings']:
		d = {}
		d['p2'] = result['p2']['value']
		d['p'] = result['p']['value']
		d['s'] = result['s']['value']
		d['o'] = result['o']['value']
		d['cType'] = result['cType']['value']
		rs.append(d)
	return rs

def getBioPortalMeasurementDesc(uri):
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
	sql = """PREFIX foaf: <http://xmlns.com/foaf/0.1/>
			PREFIX dbo: <http://dbpedia.org/ontology/>
			PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
			PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
			PREFIX rdfs2: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
			PREFIX : <http://carre.kmi.open.ac.uk/ontology/sensors.owl#>
			PREFIX carreManufacturer: <http://carre.kmi.open.ac.uk/manufacturers/>
			PREFIX carreUsers: <https://carre.kmi.open.ac.uk/users/>
			PREFIX owl: <http://www.w3.org/2002/07/owl#>

			SELECT ?cType ?s ?p  ?p2 ?o WHERE 
			{ 
			   GRAPH  <http://carre.kmi.open.ac.uk/beta> {
			   ?measurementPredicate rdfs:domain <""" + uri + """> . 
			    ?measurementPredicate rdfs:range ?measurementTypeValue .
			    ?measurementTypeValue rdfs:subClassOf ?value . 
			     ?value rdfs:subClassOf ?entity . 
			      ?entity owl:onProperty :has_measurement_type  .
			     ?entity owl:someValuesFrom ?measurementType . 
			    ?measurementType rdfs:subClassOf ?cType . 
			 }
			SERVICE <http://sparql.bioontology.org/ontologies/sparql?apikey=51d0c12d-9c18-4401-b85c-7ea4061fc0c2>
			   { 
				GRAPH ?g {
			       OPTIONAL {?s ?p ?cType }.
			      OPTIONAL { ?cType ?p2 ?o }. 
			      OPTIONAL {?s2 ?cType ?o2 }. 
				}
			   }

			} GROUP BY ?cType"""
	lld = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql")
	lld.setQuery(sql)
	lld.setReturnFormat(JSON)
	results = lld.query().convert()
	rs = []
	for result in results['results']['bindings']:
		d = {}
		d['p2'] = result['p2']['value']
		d['p'] = result['p']['value']
		d['s'] = result['s']['value']
		d['o'] = result['o']['value']
		d['cType'] = result['cType']['value']
		rs.append(d)
	return rs

def getRiskEvidenceCitations(IRI):
	import json
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF

	sql = __getRiskEvidenceCitationsQuery__(IRI)
	st = sparql(sql, "ANY_USER_HERE", "ANY_USERS_PASSWORD_HERE")
	rs = json.loads(st)
	citationIDs = []
	for item in rs:
		tmp = item['o']['value']
		tmp = tmp.replace("http://carre.kmi.open.ac.uk/citations/","")
		citationIDs.append(tmp)

	rs = []
	for id in citationIDs:
		d = getPUBMedMetadata(id)
		rs.append(d)
	return rs 

def getDataContinuous(measurement, username=None, password=None,token=None, period=365):
	import pandas as pd
	import json
	global measurements
	try:
		predicate = getPredicate(measurement)
		if predicate is None:
			return predicate

		rs = {}
		d = getStartDate(period)
		if username is None:
			import virtuoso
			username = virtuoso.getUsernameFromToken(token)
		sql = __getQuery__(predicate, d, username)
		if password is None:
			import virtuoso
			rs = virtuoso.executeSPARQL(sql, token)
			items = rs
		else:
			rs = sparql(sql, username, password)
			items = json.loads(rs)
		df = pd.DataFrame()
		for item in items:
			d = {}
			d[item['d']['value']] = item['value']['value']
			tmp = pd.DataFrame.from_dict(d.items())
			df = pd.concat([df, tmp])
		df['date'] = df[0].apply(pd.to_datetime)
		df.rename(columns={1: 'value'}, inplace=True)
		df = df[['date', 'value']]

		df['value'] = df['value'].astype(float)

		df = df.sort('date')
		
		df = df.reset_index().sort('date')
		del df['index']
	except Exception, e:
		return str(e)
	return df

def getUserManufacturers(token):
	import virtuoso
	username = virtuoso.getUsernameFromToken(token)
	if username == "":
		return None

	sql = """PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
			PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
			PREFIX carreSensors: <http://carre.kmi.open.ac.uk/ontology/sensors.owl#>
			PREFIX carreManufacturer: <http://carre.kmi.open.ac.uk/manufacturers/>
			PREFIX carreUsers: <https://carre.kmi.open.ac.uk/users/>

			SELECT DISTINCT ?manufacturer FROM <https://carre.kmi.open.ac.uk/users/""" + username + """> WHERE
			{
			        ?subject carreSensors:is_measured_by  ?manufacturer .
			}"""
	results = virtuoso.executeSPARQL(sql, token)
	rs = []
	for result in results:
		rs.append(result['manufacturer']['value'])
	return rs

def getMeasurementManufacturer(manufacturer, token):
	import virtuoso
	username = virtuoso.getUsernameFromToken(token)
	if username == "":
		return None

	manufacturer = manufacturer.replace("http://carre.kmi.open.ac.uk/manufacturers/", "")

	sql = """PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
			PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
			PREFIX carreSensors: <http://carre.kmi.open.ac.uk/ontology/sensors.owl#>
			PREFIX carreManufacturer: <http://carre.kmi.open.ac.uk/manufacturers/>
			PREFIX carreUsers: <https://carre.kmi.open.ac.uk/users/>

			SELECT * FROM <https://carre.kmi.open.ac.uk/users/""" + username + """> WHERE
			{
			        ?subject carreSensors:is_measured_by  carreManufacturer:""" + manufacturer + """ .
			        ?subject ?predicate ?object .
			        OPTIONAL { ?object carreSensors:has_value ?value }
			}"""
	results = virtuoso.executeSPARQL(sql, token)
	rs = []
	for result in results:
		d = {}
		d['predicate'] = result['predicate']['value']
		d['object'] = result['object']['value']
		if 'value' in result.keys():
			d['value'] = result['value']['value']
		d['subject'] = result['subject']['value']
		rs.append(d)
	return rs

def getAveragesDataContinuous(measurement, username, password, period=365, grouping="M"):
	df = getDataContinuous(measurement, username=username, password=password, period=period)
	df = df.set_index('date')
	davg = df.resample('M', how='mean')
	return df

def fetchCaloriesPie(username, password):
	import gviz_api
	calories = getDataContinuous("Calories", username = username,password = password)
	caloriesBMR = getDataContinuous("CaloriesBMR", username = username,password = password)
	activityCalories = getDataContinuous("ActivityCalories", username = username,password = password)

	description = {"calories": ("number", "Calories"),
			   "caloriesBMR": ("number", "CaloriesBMR"),
			   "activityCalories": ("number", "ActivityCalories")}
	data = [{"Calories": 3000, "caloriesBMR": 2000, "ActivityCalories": 1000},
			{"Calories": 5000, "caloriesBMR": 3000, "ActivityCalories": 1000}]
	data_table = gviz_api.DataTable(description)
	data_table.LoadData(data)
	json_description = data_table.ToJSCode("json_description",
							   columns_order=("calories", "caloriesBMR", "activityCalories"),
							   order_by="calories")
	json = data_table.ToJSon(columns_order=("calories", "caloriesBMR", "activityCalories"),
						   order_by="calories")
	return (json_description, json)


def sparql(q, username, password, format='JSON'):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
		import json

		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)

		sparql.setQuery(q)
		if format.lower()=='json':
			sparql.setReturnFormat(JSON)
		elif format.lower()=='turtle':
			sparql.setReturnFormat(TURTLE)
		elif format.lower()=='xml':
			sparql.setReturnFormat(XML)
		elif format.lower()=='n3':
			sparql.setReturnFormat(N3)
		else:
			return "invalid format. Select one of the following: JSON (default), XML, TURTLE, N3, RDF"
		results = sparql.query().convert()
		
	except Exception, e:
		return str(e)
	if format.lower()=='json':
		return json.dumps(results['results']['bindings'])#df.to_json()
	elif format.lower()=='turtle':
		return results
	elif format.lower()=='xml':
		return results.toxml()
	elif format.lower()=='n3':
		return results


def __sql_available_measurements2__(username):
	
	sql = """SELECT max(?datevalue) as ?lastMeasured  min(?datevalue) as ?firstMeasured ?predicate FROM <https://carre.kmi.open.ac.uk/users/""" + username + """> WHERE {
			?subject ?predicate ?object .
			?subject <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date> ?date .
			?date <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?datevalue .
			?object <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?value} 
			group by ?predicate
			order by DESC(?lastMeasured) ASC(?firstMeasured)

			"""
	return sql

def __sql_available_measurements__(username):
	
	sql = """SELECT distinct ?predicate FROM <https://carre.kmi.open.ac.uk/users/"""+username+"""> WHERE 
			{ ?subject ?predicate ?object .
			FILTER (REGEX(STR(?predicate), "http://carre.kmi.open.ac.uk/ontology/sensors.owl", "i")) 
			}
			"""
	return sql


def __sql_average_measurements(username, predicate):
	sql = """select avg(?value) as ?value WHERE
	{SELECT max(?v1) as ?value xsd:date(?d) as ?d  FROM <https://carre.kmi.open.ac.uk/users/""" + username + """> WHERE
	{
	         ?subject <"""+predicate+"""> ?object.
	         ?subject <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date> ?d1.
	         ?d1 <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?d.
             ?object <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?v1.
	}
	GROUP BY xsd:date(?d)}
	"""
	return sql


def __sql_daily_measurements__(username, d):
	d1 = str(d.year) + "-" +d.strftime('%m') + "-" + d.strftime('%d')
	sql = """SELECT max(?value) as ?value ?d ?predicate WHERE
			  {
			   SELECT max(?v1) as ?value xsd:date(?d) as ?d ?predicate FROM <https://carre.kmi.open.ac.uk/users/""" + username + """> WHERE
			  {
			       ?subject ?predicate ?object.
			       ?subject <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date> / <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?d.
			             ?object <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_value> ?v1.
			             FILTER (!regex(str(?d), "^http://carre.kmi.open.ac.uk")) .
			             FILTER (!regex(str(?v1), "^http://carre.kmi.open.ac.uk")) .
			  }
			  }
			  GROUP BY xsd:date(?d) ?predicate
			  HAVING regex(str(xsd:date(?d)),\"""" + d1 + """\")
	"""
	return sql

def __getDailyMeasurements__(username=None, password=None, token=None, d=None):
	import json
	import virtuoso
	if username is None:
		username = virtuoso.getUsernameFromToken(token)
	
	if isinstance(d, str):
		tokens = d.split("-")
		import datetime
		d = datetime.datetime(int(tokens[0]), int(tokens[1]), int(tokens[2]))

	sql = __sql_daily_measurements__(username, d)
	if password is None:
		st = virtuoso.executeSPARQL(sql, token)
		l = st
	else:
		st = sparql(sql, username, password)
		l = json.loads(st)

	rs = {}
	for item in l:
		predicate = item['predicate']['value']
		if predicate in measurements.values():
			if (predicate!="http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date") & (predicate != "http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_start_date") & (predicate != "http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_sleep_status"):
				rs[predicate] = round(float(item['value']['value']),1)

	averages = {}
	for predicate in rs.keys():
		sql = __sql_average_measurements(username, predicate)
		if password is None:
			st = virtuoso.executeSPARQL(sql, token)
			l = st
		else:
			st = sparql(sql, username, password)
			l = json.loads(st)
		averages[predicate] = round(float(l[0]['value']['value']),1)

	labels = {}
	im = {v: k for k, v in measurements.items()}
	for predicate in rs.keys():
		labels[predicate] = im[predicate]


	import pandas as pd
	df1 = pd.DataFrame(rs, index={0}).transpose()
	df1.rename(columns={0: 'daily'}, inplace=True)

	df2 = pd.DataFrame(averages, index={0}).transpose()
	df2.rename(columns={0: 'average'}, inplace=True)

	df3 = pd.DataFrame(labels, index={0}).transpose()
	df3.rename(columns={0: 'label'}, inplace=True)


	df = pd.merge(df1, df2, left_index=True, right_index=True)
	df = pd.merge(df, df3, left_index=True, right_index=True)
	return df

def __listAvailableMeasurements__(username=None, password=None, token=None):
	import json
	import virtuoso
	if username is None:
		username = virtuoso.getUsernameFromToken(token)
	sql = __sql_available_measurements__(username)
	# print sql
	if password is None:
		st = virtuoso.executeSPARQL(sql, token)
		l = st#json.loads(st)
	else:
		st = sparql(sql, username, password)
		l = json.loads(st)
	predicates = []
	for item in l:
		predicates.append(item['predicate']['value'])
	if 'http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date' in predicates:
		predicates.remove('http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_date')

	return predicates

def __listAvailableMeasurements2__(username=None, password=None, token=None):
	import json
	import virtuoso
	if username is None:
		username = virtuoso.getUsernameFromToken(token)
	sql = __sql_available_measurements2__(username)
	# print sql
	if password is None:
		st = virtuoso.executeSPARQL(sql, token)
		l = st#json.loads(st)
	else:
		st = sparql(sql, username, password)
		l = json.loads(st)
	rs = []
	for item in l:
		rs.append((item['predicate']['value'], item['firstMeasured']['value'], item['lastMeasured']['value']))

	import pandas as pd
	df = pd.DataFrame(rs)
	df.columns = ['measurement', 'firstMeasured', 'lastMeasured']
	df = df[df.measurement.isin(measurements.values())]
	df = df[~df.measurement.str.contains("date")]
	df.firstMeasured = pd.to_datetime(df.firstMeasured)
	df.lastMeasured = pd.to_datetime(df.lastMeasured)
	return df

def getAllMeasurements(username=None, password=None, token=None):
	import pandas as pd
	predicates = __listAvailableMeasurements__(username=username, password=password, token=token)
	df = pd.DataFrame()
	for predicate in predicates:
		tmp = getDataContinuous(predicate, username = username, password = password, token = token)
		if tmp is None:
			continue

		if len(tmp)==0:
			continue

		if isinstance(tmp, basestring):
			continue

		tmp.rename(columns={'value': predicate}, inplace=True)
		
		if len(df)==0:
			df = tmp
		else:
			df = pd.merge(tmp, df, left_on='date', right_on='date', how='outer')

	return df

def createICal(username=None, password=None, token=None):
	from icalendar import Calendar, Event
	import uuid
	import numpy as np
	# UIDs seem to be required. (I used the standard Python module uuid's uuid4 function. When I did not use UUIDs, or used weak UIDs, Google only fetched the file once, according to server logs.)

	df = getAllMeasurements(username, password, token)
	if 'date' in df.columns:
		df['date'] = df['date'].apply(lambda x: x.strftime('%Y%m%d'))
		df = df.set_index('date')
	
	global measurements
	im = {v: k for k, v in measurements.items()}
	columns = df.columns.tolist()
	

	cal = Calendar()
	cal.add('version', '2.0')
	cal.add('PRODID','George Gkotsis george.gkotsis@open.ac.uk//KMi, OU, UK//EN')
	
	for row in df.iterrows():
		event = Event()
		event['uid'] = str(uuid.uuid4())
		event['dtstart'] = row[0]
		d = event['dtstart'][0:4] + "-" + event['dtstart'][4:6] + "-" + event['dtstart'][6:8]
		event['dtend'] = row[0]
		summary = []
		description = []
		for measurement in columns:
			v = row[1][measurement]
			if np.isnan(v):
				continue
			summary.append(im[measurement] + " " + str(v))
			if im[measurement]== "SleepTime":
				m, s = divmod(v, 60)
				h, m = divmod(m, 60)
				if m!=0:
					description.append("Sleep Time: " + str(int(h)) + " hours, " + str(int(m))+ " minutes")
				else:
					description.append("Sleep Time: " + str(int(h)) + " hours")
			elif im[measurement]== "ActivityCalories":
				description.append("Activity: " + str(int(v))+ " Calories")
			elif im[measurement]== "Distance":
				description.append(im[measurement] + ": " + str(round(v,2)) + " kms")	
			elif im[measurement]== "Steps":
				description.append(im[measurement] + ": " + str(int(v)))
			else:
				description.append(im[measurement] + ": " + str(v))
		summary = " ".join(summary)
		description.sort()
		print description
		description = "\n".join(description)
		# print description
		event['summary'] = 'Activity Digest'
		event['description'] = description# + "<a href='http://kimmobrunfeldt.github.io/progressbar.js/'>test link</a>"
		# import requests
		# import base64
		# img = requests.get("https://carre.kmi.open.ac.uk/devices/static/icons/googlefit.png")
		# event['attach'] = base64.encodestring(img.content)
		event['URL'] = "https://carre.kmi.open.ac.uk/devices/explore/day?v="+d
		cal.add_component(event)


	return cal.to_ical()

def createHTML(url, body):
	import BeautifulSoup
	d = BeautifulSoup.BeautifulSoup()
	d.append(body)
	body = d.prettify()
	body = body.replace("\n", "<br/>")


	head = """
	<!DOCTYPE html>
	<html>
  		<head>
    	<title>""" + url +"""</title>
    	<meta charset="utf-8">
    	<meta name="viewport" content="width=device-width, initial-scale=1">
    	<link rel="stylesheet" href="http://maxcdn.bootstrapcdn.com/bootstrap/3.3.1/css/bootstrap.min.css">
  		</head>

	<body>
	<br/><br/><br/>
    <div class="container">
        <div class="col-md-12 col-lg-12 row">
	<p class="lead">
	"""

	foot = """
		</p>
		</div>
	</div>
	    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js"></script>
    	<script src="http://maxcdn.bootstrapcdn.com/bootstrap/3.3.1/js/bootstrap.min.js"></script>
  	</body>
	</html>
	"""
	html = head + body + foot




	return html

def describe(url, username, password, returnFormat="N3"):
	try:
		from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
		import json

		sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
		sparql.setHTTPAuth(DIGEST)
		sparql.setCredentials(username, password)
		sql = "DESCRIBE <" + url + "> FROM <https://carre.kmi.open.ac.uk/users/" + username + ">"
		sparql.setQuery(sql)
		# return sql
		if returnFormat=="N3":
			sparql.setReturnFormat(N3)
		elif returnFormat=="RDF":
			sparql.setReturnFormat(RDF)
		elif returnFormat=="XML":
			sparql.setReturnFormat(XML)
		elif returnFormat=="JSON":
			sparql.setReturnFormat(JSON)
		results = sparql.query().convert()

		if returnFormat=="N3":
			html = createHTML(url, results)
		else:
			return results
	except Exception, e:
		return "unknown error"

	return html
